/*----------------------------------------------------------------------------
 Author:	 Werner Boellmann
 Remarks:	 Keine Garantie auf Funktion. 
 known Problems: none
 Version:	 17.11.2008
 Description:	 NTP-Zeit Umwandlung 

 Dieses	Programm ist freie Software. Sie können	es unter den Bedingungen der
 GNU General Public License, wie von der Free Software Foundation veröffentlicht,
 weitergeben und/oder modifizieren, entweder gemäß Version 2 der Lizenz	oder
 (nach Ihrer Option) jeder späteren Version.

 Die Veröffentlichung dieses Programms erfolgt in der Hoffnung,
 daß es	Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE,
 sogar ohne die	implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT
 FÜR EINEN BESTIMMTEN ZWECK. Details finden Sie	in der GNU General Public License.

 Sie sollten eine Kopie	der GNU	General	Public License zusammen	mit diesem
 Programm erhalten haben.
 Falls nicht, schreiben	Sie an die Free	Software Foundation,
 Inc., 59 Temple Place,	Suite 330, Boston, MA 02111-1307, USA.
------------------------------------------------------------------------------*/

#include "timeutil.h"

#if USE_NTP

/*    A C H T U N G : 
      In ntp.c muss in der Funtion ntp_get() die Zeile

      ntp->rx_timestamp += GMT_TIME_CORRECTION; //  UTC +1h

      auskommentert werden !!!
*/


// Einige Hilfsfunktionen zur Laufzeitoptimierung

// --------------------------------------------------------------------------------
// Eine unsigned ldiv Funtion. Siehe <stdlib.h>

/** Result type for function uldiv(). */
typedef struct {
	unsigned long quot;                  /**< The Quotient. */
	unsigned long rem;                   /**< The Remainder. */
} uldiv_t;

// --------------------------------------------------------------------------------
/**
     The uldiv() function computes the value \c num/denom and returns
     the quotient and remainder in a structure named \c ldiv_t that
     contains two long integer members named \c quot and \c rem.
*/
extern uldiv_t uldiv(unsigned long __num, unsigned long __denom) __asm__("__udivmodsi4") __ATTR_CONST__;

// --------------------------------------------------------------------------------
// Namen der Wochentage:
static PROGMEM char so_s[] = "Sun";
static PROGMEM char mo_s[] = "Mon";
static PROGMEM char di_s[] = "Tue";
static PROGMEM char mi_s[] = "Wed";
static PROGMEM char do_s[] = "Thu";
static PROGMEM char fr_s[] = "Fri";
static PROGMEM char sa_s[] = "Sat";

// --------------------------------------------------------------------------------
// Tabelle der Monatsnamen: Monatsnummer mudulo 12 -> Dezember = 0
static PROGMEM char jan_s[] = "Jan";
static PROGMEM char feb_s[] = "Feb";
static PROGMEM char mar_s[] = "Mar";
static PROGMEM char apr_s[] = "Apr";
static PROGMEM char mai_s[] = "May";
static PROGMEM char jun_s[] = "Jun";
static PROGMEM char jul_s[] = "Jul";
static PROGMEM char aug_s[] = "Aug";
static PROGMEM char sep_s[] = "Sep";
static PROGMEM char okt_s[] = "Oct";
static PROGMEM char nov_s[] = "Nov";
static PROGMEM char dez_s[] = "Dec";

// --------------------------------------------------------------------------------
PROGMEM PGM_P dayNameTab_short[] = { so_s, mo_s, di_s, mi_s, do_s, fr_s, sa_s };

// --------------------------------------------------------------------------------
PROGMEM PGM_P monthNameTab_short[] =
{ jan_s, feb_s, mar_s, apr_s, mai_s, jun_s, jul_s, aug_s, sep_s, okt_s, nov_s, dez_s };

// -----------------------------------------------------------------------------------
/*
 * Ist gerade Sommerzeit ?
 *     1 = JA
 *     0 = nein
 */
uint8_t isDST( struct tm *ptm )
{
	if( ptm->tm_mon < 1 || ptm->tm_mon > 9 )	// month 1, 2, 11, 12
		return 0;			// -> Winter

	// after last Sunday 2:00 // weekday: 0 = sunday 1 = monday
	if( (ptm->tm_mday - ptm->tm_wday) >= 25 && (ptm->tm_wday || ptm->tm_hour >= 2) )
	{
		if( ptm->tm_mon == 9 )		// October -> Winter
			return 0;
	}
	else					// before last Sunday 2:00
	{
		if( ptm->tm_mon == 2 )		// March -> Winter
		    return 0;
	}

	return 1;
}

// -----------------------------------------------------------------------------------

/* 
 * 16. Nov 2008 Werner Boellmann
 * The following code is based on ...
 */
/* a small POSIX-like date implementation
 *
 * (C) 2004 Ruediger Sonderfeld <kingruedi@c-plusplus.de>
 * 2005/11/03 Modified by Nikola Vladov
 * http://riemann.fmi.uni-sofia.bg/vladov/ftp/
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


static struct tm *ntptime(const uint32_t t, uint8_t local) 
{
  static struct tm tm;
  unsigned long day, mon, year, yday, tod;

  uldiv_t ld = uldiv(t, 86400L);	// day = ld.quot; tod = ld.rem;
  tod = (unsigned long)ld.rem;		// tod = (unsigned long)(t) % 86400L;
  day = (unsigned long)ld.quot;		// day = (unsigned long)(t) / 86400L;

  tm.tm_wday = ((day+1) % 7);

  ld = uldiv(tod, 60);
  tm.tm_sec  = ld.rem; tod = ld.quot; 	//tm.tm_sec = tod%60; tod /= 60;

  ld = uldiv(tod, 60);
  tm.tm_min  = ld.rem;  		//tm.tm_min  = tod%60; 
  tm.tm_hour = ld.quot; 		//tm.tm_hour = tod/60;

  year = (2 + 4 * day) / 1461;

  day +=  671;
  day %= 1461;           /* day 0 is march 1, 1972 */

  yday = (day < 306) ? 1 : 0;
  if (day == 1460)
  { 
	day  = 365; 
	yday = 59; 
  }
  else 
  { 
        day  %= 365;   
        yday += (day + 59) % 365; 
  }

  day *= 10;
  mon  = (day + 5) / 306;
  day  = day + 5 - 306 * mon;
  day /= 10;
  if (mon >= 10) 
	mon -= 10;
  else       /* day 0,1,30, mon 0..11,  year 1970=0,1,2 */
	mon += 2;

  tm.tm_year = year + 1900;
  tm.tm_mon  = mon;
  tm.tm_mday = day + 1;
  if((tm.tm_dls = isDST( &tm )) && local)
  {
	// Add daylight savings offset 
	return ntptime( t + DAYLIGHT_TIME_CORRECTION, 0 );
  }
  tm.tm_yday = yday;
  return &tm;
}

struct tm *localntptime(const uint32_t t) 
{
	return ntptime(t + GMT_TIME_CORRECTION, 1);
}

struct tm *gmntptime(const uint32_t t) 
{
	return ntptime(t, 0);
}

#endif //USE_NTP





