/*----------------------------------------------------------------------------
 Author:	 Werner Böllmann
 Remarks:	 Keine Garantie auf Funktion. 
 known Problems: none
 Version:	 17.11.2008
 Description:	 NTP-Zeit Umwandlung

 Dieses	Programm ist freie Software. Sie können	es unter den Bedingungen der
 GNU General Public License, wie von der Free Software Foundation veröffentlicht,
 weitergeben und/oder modifizieren, entweder gemäß Version 2 der Lizenz	oder
 (nach Ihrer Option) jeder späteren Version.

 Die Veröffentlichung dieses Programms erfolgt in der Hoffnung,
 daß es	Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE,
 sogar ohne die	implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT
 FÜR EINEN BESTIMMTEN ZWECK. Details finden Sie	in der GNU General Public License.

 Sie sollten eine Kopie	der GNU	General	Public License zusammen	mit diesem
 Programm erhalten haben.
 Falls nicht, schreiben	Sie an die Free	Software Foundation,
 Inc., 59 Temple Place,	Suite 330, Boston, MA 02111-1307, USA.
------------------------------------------------------------------------------*/
#include "config.h"

#if USE_NTP
#ifndef	_NTPUTIL_H
	#define	_NTPUTIL_H

#include "ntp.h"

/*    A C H T U N G : 
      In ntp.c muss in der Funtion ntp_get() die Zeile

      ntp->rx_timestamp += GMT_TIME_CORRECTION; //  UTC +1h

      auskommentert werden !!!
*/
// --------------------------------------------------------------------------------
// Sommerzeit Offset 
#define	DAYLIGHT_TIME_CORRECTION 3600	// 1 Stunde in Sekunden

// --------------------------------------------------------------------------------
struct tm {
  uint16_t tm_year;	// 1900...
  uint8_t  tm_mon;	// Monate seit dem 1.1. (z.B. 10=November)
  uint8_t  tm_mday;     // Tag des Monats
  uint16_t tm_yday;	// Tag des Jahres
  uint8_t  tm_hour;	
  uint8_t  tm_min; 
  uint8_t  tm_sec;
  uint8_t  tm_wday;	// Wochentag. 0=So. ... 6=Sa.
  uint8_t  tm_dls;	// Sommerzeit-Flag
};

// --------------------------------------------------------------------------------
// Wandelt den von der NTP Funktion gelieferten Zeitstempel
// in die tt.mm.yyyy HH:mm:ss GMT Darstellung der struct tm um.
// Achtung: Die Funktion ist (wie gmtime) nicht reentrant!  
extern struct tm *gmntptime(const uint32_t t);


// --------------------------------------------------------------------------------
// Wandelt den von der NTP Funktion gelieferten Zeitstempel
// in die tt.mm.yyyy HH:mm:ss MEZ/MESZ Darstellung der struct tm um. 
// Achtung: Die Funktion ist (wie localtime) nicht reentrant!  
extern struct tm *localntptime(uint32_t t);

// --------------------------------------------------------------------------------
// Tabelle der Monate im PROGMEM 
extern PROGMEM PGM_P monthNameTab_short[];

// --------------------------------------------------------------------------------
// Tabelle der Wochentage im PROGMEM 
extern PROGMEM PGM_P dayNameTab_short[];

// --------------------------------------------------------------------------------
// Makro liefert einen PROGMEM * auf den Wochentag zurück.
#define dayname_short(wday) 	((PGM_P)pgm_read_word(&dayNameTab_short[(wday)%7]))

// --------------------------------------------------------------------------------
// Makro liefert einen PROGMEM * auf den Monat zurück. 0=Januar ... 11 = Dezember
// Wie in struct tm zurückgeliefert.
#define monthname_short(month) 	((PGM_P)pgm_read_word(&monthNameTab_short[(month)%12]))

// --------------------------------------------------------------------------------
// Ist dieses Jar ein Schaltjahr
static inline uint8_t isLeapYear( uint16_t year ) __attribute__((__always_inline__));
static inline uint8_t isLeapYear( uint16_t year )
{
    return( ( ( ( year % 4 ) == 0 ) &&
              ( ( year % 100 ) != 0 ) )
              || ( ( year % 400 ) == 0 ) );
}
// --------------------------------------------------------------------------------
// Ist gerade Sommerzeit
uint8_t isDST( struct tm *ptm );

#endif
#endif //USE_NTP

